# BUILD NOTE:
# until we get binutils with https://sourceware.org/bugzilla/show_bug.cgi?id=28138 fixed
# build requires >1024 available file descriptors (4096 seems sufficient)
#
# TODO
# - unpackaged files
#   /usr/share/doc/node/gdbinit

# Conditional build:
%bcond_without	system_brotli	# system brotli
%bcond_without	system_uv	# system uv
%bcond_with	http_parser	# system http-parser and llhttp

%define		_enable_debug_packages	0

# NOTES:
# - https://nodejs.org/en/download/releases/

# NODE_MODULE_VERSION refers to the ABI (application binary interface) version
# number of Node.js, used to determine which versions of Node.js compiled C++
# add-on binaries can be loaded in to without needing to be re-compiled. It
# used to be stored as hex value in earlier versions, but is now represented as
# an integer.
%define		node_module_version	127
Summary:	Asynchronous JavaScript Engine
Summary(pl.UTF-8):	Asynchroniczny silnik JavaScriptu
Name:		nodejs
# 22.x LTS - https://github.com/nodejs/Release
# Active start: 2024-10-29
# Maintenance start: October 2025
# Maintenance end: April 2027
Version:	22.17.0
Release:	1
License:	BSD and MIT and Apache v2.0 and GPL v3
Group:		Development/Languages
Source0:	https://nodejs.org/download/release/latest-v22.x/node-v%{version}.tar.xz
# Source0-md5:	49ea94e07c7409174b7f98c6a0f723a4
# force node to use /usr/lib/node as the systemwide module directory
Patch0:		%{name}-libpath.patch
# use /usr/lib64/node as an arch-specific module dir when appropriate
Patch1:		%{name}-lib64path.patch
Patch2:		0001-Remove-unused-OpenSSL-config.patch
Patch3:		arm-yield.patch
Patch4:		cflags.patch
Patch5:		abseil-flags.patch
URL:		https://nodejs.org/
BuildRequires:	c-ares-devel >= 1.17.2
BuildRequires:	gcc >= 6:6.3
%if %{with http_parser}
BuildRequires:	http-parser-devel >= 2.9.3
BuildRequires:	llhttp-devel >= 2.1.3
%endif
%ifarch mips mipsel mips64 mips64el ppc %{arm}
BuildRequires:	libatomic-devel
%endif
%{?with_system_brotli:BuildRequires:	libbrotli-devel >= 1.0.9}
BuildRequires:	libicu-devel >= 69.1
BuildRequires:	libstdc++-devel >= 6:4.8
%{?with_system_uv:BuildRequires:	libuv-devel >= 1.42.0}
BuildRequires:	nghttp2-devel >= 1.42.0
BuildRequires:	openssl-devel >= 1.0.1
BuildRequires:	pkgconfig
BuildRequires:	python3 >= 1:3.6
BuildRequires:	python3-modules >= 1:3.6
BuildRequires:	rpm >= 4.4.9-56
BuildRequires:	rpm-build >= 4.6
BuildRequires:	rpmbuild(macros) >= 1.752
BuildRequires:	tar >= 1:1.22
BuildRequires:	sed >= 4.0
BuildRequires:	xz
BuildRequires:	zlib-devel >= 1.2.11
Requires:	c-ares >= 1.17.1
Requires:	ca-certificates
%ifarch %{ix86}
Requires:	cpuinfo(sse2)
%endif
%{?with_http_parser:Requires:	http-parser >= 2.9.3}
%{?with_system_brotli:Requires:	libbrotli >= 1.0.9}
%{?with_system_uv:Requires:	libuv >= 1.42.0}
Requires:	nghttp2-libs >= 1.42.0
Requires:	zlib >= 1.2.11
Provides:	nodejs(engine) = %{version}
Provides:	nodejs(module-version) = %{node_module_version}
Obsoletes:	nodejs-waf < 0.9
Obsoletes:	systemtap-nodejs < 22.4.0
ExclusiveArch:	%{ix86} %{x8664} %{arm} aarch64
BuildRoot:	%{tmpdir}/%{name}-%{version}-root-%(id -u -n)

%define		sover	%(echo %{version} | cut -d. -f2)
# add macro, so adapter won't replace it back literal
%define		doc_ver	%{version}

%description
Node.js is a platform built on Chrome's JavaScript runtime for easily
building fast, scalable network applications. Node.js uses an
event-driven, non-blocking I/O model that makes it lightweight and
efficient, perfect for data-intensive real-time applications that run
across distributed devices.

%description -l pl.UTF-8
Node.js to platforma zbudowana w opacriu o silnik JavaScriptu
przeglądarki Chrome, służąca do tworzenia szybkich, skalowalnych
aplikacji sieciowych. Node.js wykorzystuje nieblokujący model
wejścia/wyjścia sterowany zdarzeniami, dzięki czemu jest lekki i
wydajny, dobrze nadający się do aplikacji przetwarzających duże
ilości danych w czasie rzeczywistym, uruchamianych na rozproszonych
urządzeniach.

%package devel
Summary:	Development headers for nodejs
Summary(pl.UTF-8):	Pliki nagłówkowe nodejs
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}
Requires:	gcc
%{?with_http_parser:Requires:	http-parser-devel >= 2.9.3}
Requires:	libstdc++-devel
%{?with_system_uv:Requires:	libuv-devel >= 1.42.0}
Requires:	openssl-devel
Requires:	zlib-devel >= 1.2.11

%description devel
Development headers for nodejs.

%description devel -l pl.UTF-8
Pliki nagłówkowe nodejs.

%package doc
Summary:	Documentation for Node.js engine
Summary(pl.UTF-8):	Dokumentacja silnika Node.js
Group:		Documentation
URL:		https://nodejs.org/dist/v%{doc_ver}/docs/api
BuildArch:	noarch

%description doc
Node.js is a server-side JavaScript environment that uses an
asynchronous event-driven model. Node's goal is to provide an easy way
to build scalable network programs.

This package contains the documentation for Node.js.

%description doc -l pl.UTF-8
Node.js to serwerowe środowisko JavaScriptu wykorzystujące
asynchroniczny model sterowany zdarzeniami. Celem Node jest
zapewnienie łatwego sposobu tworzenia skalowalnych programów
sieciowych.

Ten pakiet zawiera dokumentację Node.js.

%prep
%setup -q -n node-v%{version}
%if "%{_lib}" == "lib64"
%patch -P1 -p1
%else
%patch -P0 -p1
%endif
%patch -P2 -p1
%patch -P3 -p1
%patch -P4 -p1
%patch -P5 -p1

grep -r '#!.*env python' -l . | xargs %{__sed} -i -e '1 s,#!.*env python$,#!%{__python3},'

%{?with_system_brotli:%{__rm} -r deps/brotli}
%{__rm} -r deps/cares
%if %{with http_parser}
%{__rm} -r deps/http_parser
%{__rm} -r deps/llhttp
%endif
%{__rm} -r deps/icu-small
%{__rm} -r deps/nghttp2
%{__rm} -r deps/npm
%{__rm} -r deps/openssl
%{?with_system_uv:%{__rm} -r deps/uv}
%{__rm} -r deps/zlib

%build
ver=$(awk '/#define NODE_MODULE_VERSION [0-9]+/{print $3}' src/node_version.h)
test "$ver" = "%{node_module_version}"

# CC used only to detect if CC is clang, not used for compiling
CC="%{__cc}" \
CXX="%{__cxx}" \
GYP_DEFINES="soname_version=%{sover}" \
./configure \
	--prefix=%{_prefix} \
	--libdir=%{_lib} \
	--openssl-use-def-ca-store \
	--shared \
	%{?with_system_brotli:--shared-brotli} \
	--shared-cares \
	%{?with_http_parser:--shared-http-parser} \
	%{?with_system_uv:--shared-libuv} \
	--shared-nghttp2 \
	--shared-openssl \
	--shared-zlib \
	--with-intl=system-icu \
	--without-corepack \
	--without-npm

# add LFS defines from libuv (RHBZ#892601)
# CXXFLAGS must be exported, as it is needed for make, not gyp
CXXFLAGS="%{rpmcxxflags} -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64 -fPIC" \
LDFLAGS="%{rpmldflags}" \
PATH="$(pwd)/out/tools/bin:$PATH" \
%{__make} -C out \
	BUILDTYPE=Release \
	V=1

%install
rm -rf $RPM_BUILD_ROOT

%{__python3} tools/install.py --dest-dir "$RPM_BUILD_ROOT" --prefix "%{_prefix}" install

ln -s libnode.so.%{node_module_version} $RPM_BUILD_ROOT%{_libdir}/libnode.so

echo '.so man1/node.1' > $RPM_BUILD_ROOT%{_mandir}/man1/nodejs.1

install -d $RPM_BUILD_ROOT%{_includedir}/node
cp -p src/*.h $RPM_BUILD_ROOT%{_includedir}/node

# install for node-gyp
install -d $RPM_BUILD_ROOT%{_usrsrc}/%{name}
cp -p common.gypi $RPM_BUILD_ROOT%{_usrsrc}/%{name}
ln -s %{_includedir}/node $RPM_BUILD_ROOT%{_usrsrc}/%{name}/src

# for compat of fedora derivered scripts (shebangs)
ln -s node $RPM_BUILD_ROOT%{_bindir}/nodejs

# globally installed node modules (noarch)
install -d $RPM_BUILD_ROOT%{_prefix}/lib/node_modules

# default searchpaths
install -d $RPM_BUILD_ROOT{%{_libdir},%{_prefix}/lib}/node

# create pkgconfig
install -d $RPM_BUILD_ROOT%{_pkgconfigdir}
cat <<'EOF' > $RPM_BUILD_ROOT%{_pkgconfigdir}/%{name}.pc
version=%{version}
prefix=%{_prefix}
libdir=${prefix}/%{_lib}
includedir=${prefix}/include/node

Name: nodejs
Description: Evented I/O for V8 JavaScript.
Version: ${version}
Cflags: -I${includedir}
EOF

# install documentation
install -d $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}
cp -a doc/api/* $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}
%{__rm} $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}/*.md
%{__rm} $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}/*.json

%clean
rm -rf $RPM_BUILD_ROOT

%post	-p /sbin/ldconfig
%postun -p /sbin/ldconfig

%files
%defattr(644,root,root,755)
%doc CHANGELOG.md LICENSE GOVERNANCE.md README.md SECURITY.md
%attr(755,root,root) %{_bindir}/node
%attr(755,root,root) %{_bindir}/nodejs
%attr(755,root,root) %{_libdir}/libnode.so.%{node_module_version}
%if "%{_lib}" != "lib"
%dir %{_libdir}/node
%endif
%dir %{_prefix}/lib/node
%dir %{_prefix}/lib/node_modules
%{_mandir}/man1/node.1*
%{_mandir}/man1/nodejs.1*

%files devel
%defattr(644,root,root,755)
%attr(755,root,root) %{_libdir}/libnode.so
%{_includedir}/node
%{_pkgconfigdir}/nodejs.pc
%{_usrsrc}/%{name}

%files doc
%defattr(644,root,root,755)
%doc %{_docdir}/%{name}-doc-%{version}
%changelog
* Thu Jun 26 2025 PLD Linux Team <feedback@pld-linux.org>
- For complete changelog see: http://git.pld-linux.org/?p=packages/nodejs.git;a=log;h=master

* Wed Jun 25 2025 Jan Palus <atler@pld-linux.org> 68ca27f
- up to 22.17.0

* Tue Jun 17 2025 Jan Palus <atler@pld-linux.org> d380d6b
- upstream fix to correct flags used for internal abseil

* Tue Jun 10 2025 Jan Palus <atler@pld-linux.org> 7aa74cf
- R: cpuinfo(sse2) on %{ix86}

* Sun May 25 2025 Jakub Bogusz <qboosh@pld-linux.org> ea33a40
- 22.16.0 fails during python3-furo build for me

* Fri May 23 2025 Jan Palus <atler@pld-linux.org> 941e51b
- up to 22.16.0

* Fri May 16 2025 Jan Palus <atler@pld-linux.org> b09faab
- up to 22.15.1 (fixes CVE-2025-23165 CVE-2025-23166)

* Thu May 01 2025 Jan Palus <atler@pld-linux.org> 926727a
- up to 22.15.0

* Fri Mar 07 2025 Jan Palus <atler@pld-linux.org> c933812
- up to 22.14.0

* Tue Feb 11 2025 Jan Rękorajski <baggins@pld-linux.org> 9af4f88
- don't override opt and arch flags

* Fri Jan 17 2025 Jan Palus <atler@pld-linux.org> 284052b
- icu 76 rebuild

* Wed Jan 15 2025 Jakub Bogusz <qboosh@pld-linux.org> 0d76ec0
- use patch -P

* Wed Jan 15 2025 Jan Palus <atler@pld-linux.org> 04e1b0c
- up to 22.13.0

* Sun Aug 25 2024 Jan Palus <atler@pld-linux.org> 8c8b8be
- no yield instruction in base armv6

* Sun Aug 25 2024 Jan Palus <atler@pld-linux.org> 5fb2041
- drop package information for systemtap-nodejs

* Sun Aug 25 2024 Jan Palus <atler@pld-linux.org> 4f8bf2d
- up to 22.7.0

* Fri Jul 05 2024 Jan Rękorajski <baggins@pld-linux.org> 23d6d08
- up to 22.4.0

* Fri Oct 13 2023 Jan Palus <atler@pld-linux.org> b244cdd
- up to 16.20.2 (fixes CVE-2023-32002 CVE-2023-32006 CVE-2023-32559)

* Thu Jul 06 2023 Jakub Bogusz <qboosh@pld-linux.org> 1ec00a1
- .tar.xz dependencies

* Thu Jul 06 2023 Jakub Bogusz <qboosh@pld-linux.org> ebb4387
- updated to 16.20.1

* Sun Jul 02 2023 Jan Palus <atler@pld-linux.org> 5801f7c
- icu rebuild

